@@ -23,28 +23,26 @@ class Exam(object):
 
     @before
     def setup_patchers(self):
-        # Run the patcher function, and set the results
-        for wrapped in self.attrs_of_type(patcher.wrapper):
-            ptchr = wrapped(self)
-            setattr(self, wrapped.func.__name__, ptchr.start())
-            self.addCleanup(ptchr.stop)
+        for attr, patchr in self.attrs_of_type(patcher):
+            patch_object = patchr.build_patch(self)
+            setattr(self, attr, patch_object.start())
+            self.addCleanup(patch_object.stop)
 
     def attrs_of_type(self, kind):
         for base in inspect.getmro(type(self)):
-            for value in vars(base).values():
+            for attr, value in vars(base).items():
                 if type(value) is kind:
-                    yield value
+                    yield attr, value
 
     def setUp(self):
-        # Run each before
-        for value in self.attrs_of_type(before):
+        for _, value in self.attrs_of_type(before):
             value(self)
 
     def tearDown(self):
-        for value in self.attrs_of_type(after):
+        for _, value in self.attrs_of_type(after):
             value(self)
 
     def run(self, *args, **kwargs):
-        generators = (value(self) for value in self.attrs_of_type(around))
+        generators = (value(self) for _, value in self.attrs_of_type(around))
         with MultipleGeneratorsContextManager(*generators):
             super(Exam, self).run(*args, **kwargs)
